/**
 *
 * Project: Eight Track, File: TrackManager.java
 *
 * Copyright 2019-2019 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.audio;

import java.lang.reflect.Field;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.function.Consumer;
import com.sedmelluq.discord.lavaplayer.player.AudioPlayerManager;
import com.sedmelluq.discord.lavaplayer.player.DefaultAudioPlayer;
import com.sedmelluq.discord.lavaplayer.player.DefaultAudioPlayerManager;
import com.sedmelluq.discord.lavaplayer.source.AudioSourceManagers;
import com.sedmelluq.discord.lavaplayer.track.AudioTrack;
import com.sedmelluq.discord.lavaplayer.track.AudioTrackEndReason;

public class TrackManager extends DefaultAudioPlayer  {
  
  private LinkedBlockingDeque<String> queue;
  private AudioPlayerManager playerManager;
  private TrackScheduler trackScheduler;
  private String currentItem;
  
  public TrackManager() {
    super(new DefaultAudioPlayerManager());
    this.queue = new LinkedBlockingDeque<String>();
    this.trackScheduler = new TrackScheduler();
    
    addListener(trackScheduler);
    
    try {
        Field f = this.getClass().getSuperclass().getDeclaredField("manager");
        f.setAccessible(true);
        this.playerManager = (AudioPlayerManager) f.get(this);
    } catch (SecurityException | NoSuchFieldException | IllegalArgumentException | IllegalAccessException e) {
      throw new IllegalArgumentException(e);
    }
    
    AudioSourceManagers.registerLocalSource(this.playerManager);
  }
  
  public void queueLast(String track) {
    queue.addLast(track);
  }
  
  public void queueNext(String track) {
    queue.addFirst(track);
  }
  
  public boolean hasTracks() {
    return queue.peek() != null && !queue.isEmpty();
  }
  
  public String getCurrentItem() {
    return currentItem;
  }
  
  public AudioTrack play() {
    if (!queue.isEmpty()) {
      try {
        currentItem = queue.poll();
        playerManager.loadItem(currentItem, trackScheduler).get();
        return trackScheduler.getNextTrack();
      } catch (InterruptedException | ExecutionException e) {}
    }
    return null;
  }
  
  public void setFinishedCallback(Consumer<AudioTrackEndReason> callback) {
    trackScheduler.setFinishedCallback(callback);
  }
  
  public void clearTracks() {
    queue.clear();
    trackScheduler.clearTracks();
  }
}
