/**
 *
 * Project: Eight Track, File: AbstractEvent.java
 *
 * Copyright 2019-2020 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.events;

import java.util.List;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.exceptions.ErrorResponseException;
import net.dv8tion.jda.api.requests.ErrorResponse;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.handlers.ReactionHandler;

public abstract class AbstractEvent {
  
  private String command;

  protected AbstractEvent(String command) {
    this.command = command;
  }
  
  public String getCommand() {
    return this.command;
  }
  
  public abstract void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands);
  
  
  public abstract String help();
  
  protected boolean isAdmin(GuildMessageReceivedEvent event) {
    if (!event.getMessage().getMember().hasPermission(Permission.ADMINISTRATOR, Permission.MANAGE_SERVER)) {
      event.getChannel().sendMessage(String.format("Sorry <@%s> i can't do that. *psst: you have to have manage server, or administrator role*", event.getMember().getIdLong())).queue();
      return false;
    }
    return true;
  }

  protected boolean checkCommands(GuildMessageReceivedEvent event, List<String> commands, String msg) {
    return checkCommands(event, commands, 0, msg);
  }

  protected boolean checkCommands(GuildMessageReceivedEvent event, List<String> commands, int minParams, String msg) {
    if (commands == null || commands.isEmpty() || commands.size() < minParams) {
      event.getChannel().sendMessage(msg).queue();
      return false;
    }
    return true;
  }

  protected void sendMessage(GuildMessageReceivedEvent event, String msg) {
    sendMessage(event, msg, false);
  }
  
  protected void sendMessage(GuildMessageReceivedEvent event, String msg, boolean wait) {
    if (!wait) {
      event.getChannel().sendMessage(msg).queue();
    } else {
      event.getChannel().sendMessage(msg).complete();
    }
  }
  
  protected void sendMessage(GuildMessageReceivedEvent event, String msgFmt, Object... args) {
    sendMessage(event, String.format(msgFmt, args));
  }
  
  protected void sendMessage(GuildMessageReceivedEvent event, boolean wait, String msgFmt, Object... args) {
    sendMessage(event, String.format(msgFmt, args), wait);
  }

  protected void deleteMessage(Message msg, ReactionHandler reactionHandler, Long guildId) {
    try {
      msg.delete().complete();
      if (reactionHandler != null) {
        reactionHandler.removeReactionListener(guildId, msg.getIdLong());
      }
    } catch (ErrorResponseException ex) {
      if (ex.getErrorResponse() != ErrorResponse.UNKNOWN_MESSAGE) throw ex;
    }
  }

}
